#!/usr/bin/perl

use warnings;
use strict;
use v5.10;
use Cwd();
use Getopt::Std;
use Time::Piece;
use File::Spec;
use File::Glob ':glob';
use Date::Calc qw(Delta_Days);
use HTML::Entities;
use File::Basename;

sub debug;

my $version = '1.3';

my $shntool = $^O eq 'MSWin32' ? 'shntool.exe' : 'shntool';
#my $shntool = '/opt/local/bin/shntool';
#my $shntool = 'C:\Program Files (x86)\shntool.exe';

# format of date shown in Album as part of venue (strftime format)
my $venue_date_fmt = '%B %d, %Y';

# c=convert shn to flac
# d=debug
# f=force overwrite of flac files
# m=don't write mpl
# v=verbose (output album info)
my $opt_letters = "cdfmv";

my (%opts, %tags, %bands, $cwd);

my $prog = basename($0);

getopts($opt_letters, \%opts);

die "Usage: $prog [-$opt_letters] infofile\nStopped"  if @ARGV != 1;

-e $ARGV[0] or die "$!: $ARGV[0]\nStopped";

my ($info_file_base, $info_dir, $suffix) = fileparse($ARGV[0], qr/\.[^.]*/);
my $info_file = $info_file_base . $suffix;

chdir $info_dir or die "Unable to change directory to $info_dir\n$!.\nStopped";

$cwd = Cwd::getcwd() and debug $cwd;

do_init($info_file);

my ($datepat_mdy, $datepat_ymd, $venue_date);
($tags{'date'}, $venue_date, $datepat_mdy, $datepat_ymd) = date_from_filename($info_file);

$tags{'date'} or 
    die 'Unable to obtain date from ', $info_file, "\nStopped";

# slurp the entire file
{
local $/ = undef;
shift @ARGV; push @ARGV, $info_file;
$_ = <>;
}

die "Read 0 bytes from info file $info_file"  unless $_;

s/\r//g;		# clean dos line endings
s/[ \t]*$//gm;		# clean trailing spaces

{
    my ($year,$month,$day);

    # Search for venue
    if (s/^(.+?)\n\n//s) {
	debug "Venue method 1: ", $1;
	$tags{'venue'} = $1;

	if ($tags{'venue'} =~ s/$datepat_mdy//m or
	    $tags{'venue'} =~ s/$datepat_ymd//m) {
	    ($year, $month, $day) = @+{'year','month','day'};
	    debug "date found: Year: $year, Month: $month, Day: $day";
	}
	$tags{'venue'} =~ s/^\n//;

	if ($tags{'artist'}) {
	    # check for artist match in venue, in case additional artists are present
	    if ($tags{'venue'} =~ s/^(.*\b\Q$tags{'artist'}\E\b.*$)\n//im) {
		debug "Matching artist found in venue: ", $1;
		$tags{'artist'} = $1;
	    }
	}
	else {
	    # assume artist is first line in venue, if not already found from file abbr
	    $tags{'venue'} =~ s/^(.*?)\n//;
	    $tags{'artist'} = $1;
	}

    }
    # Try using the filename's date as the end-of-venue marker
    elsif (/^(?<venue>.+?)$datepat_mdy(?<post>.*)$/sm or
	   /^(?<venue>.+?)$datepat_ymd(?<post>.*)$/sm) {

	debug "Venue method 2: ", $+{venue};
	($year, $month, $day) = @+{'year','month','day'};
	debug "date removed: Year: $year, Month: $month, Day: $day";
	$tags{'venue'} = $+{venue};
	$_ = $+{post};
    }
    else {
	die "Can't determine venue - is this a valid info file?\n", $info_file;
    }

    compare_dates_venue_filename($year,$month,$day);
}
    
# Use the first line from venue to constuct a friendly album title
$tags{'album'} = join(' ', 'Live in', (split(/\n/, $tags{'venue'}, 2))[0], $venue_date);

# tracks - use number of discovered files as a hint
my $ntrackswanted = exists $tags{'filenames'} ? scalar(keys %{$tags{'filenames'}}) : 0;
debug "Want N tracks: ", $ntrackswanted, "\n";

$tags{'comment'} = get_tracks($_, $ntrackswanted);

print_album()	if $opts{'v'};
do_convert()	if $opts{'c'};
write_mpl()	unless $opts{'m'};


sub get_tracks {
    my ($line, $nwanted) = @_;

    my $ret = undef;
    my $disc=1;
    my ($t, $name, $i);

    debug "Trying track heuristic 1";
    for (split(/\n/, $line)) {

	if (/^\s*(?:dis[ck])\s+([\dIVX]+)/i) {
	    $disc = $tags{'disc'} = roman2int($1);
	    debug "Disc: ", $disc;
	}
	elsif (/^d\d+t\d+\b/...!/^d\d+t\d+\b/ and $_) {
	    ($tags{'disc'},$t,$name) = /^d0*(\d+)t0*(\d+)[-.\s]*(.*)$/;
	    $name =~ s/\s{2,}\d+[:.]\d+$//;	# remove possible trailing duration stamp (e.g. 7:43)
	    debug "Disc- $tags{'disc'}, Track '$t' => $name";
	    push @{$tags{'tracks'}}, "d$tags{'disc'}t${t}_$name";
	}
	elsif (/^0*1\b/.../^0*$nwanted\b/ and /^\d+\b/) {
	    ($t,$name) = /^0*(\d+)[-.\s]*(.*)$/;
	    $name =~ s/\s{2,}\d+[:.]\d+$//;	# remove possible trailing duration stamp (e.g. 7:43)
	    debug "Track $t => $name";
	    push @{$tags{'tracks'}}, "d${disc}t${t}_$name";
	}
	else {
	    debug "Skipping ", $_;
	    $ret .= "$_\n";
	}
    }
    return $ret if exists $tags{'tracks'};

    debug "Trying track heuristic 2";
    while ($line) {
	if ($line =~ s/^\s*dis[ck]\s+([\dIVX]+)[^\n]+\n(.*?)(?:\n\n|\Z)//ims) {
	    $tags{'disc'} = roman2int($1);
	    debug "Disc: ", $tags{'disc'};
	    debug "FOUND TRACKS:\n'$2'";
	    $i = 1;
	    for (split(/\n/, $2)) {
		debug "Track $i => $_";
		push @{$tags{'tracks'}}, "d$tags{'disc'}t${i}_$_";
		$i++;
	    }
	}
	else {
	    debug "Heuristic 2 failed";
	    last;
	}
	debug "LINE now: '$line'";
    }
    if (exists $tags{'tracks'}) {
	debug "Heuristic 2 found tracks: returning: '$line'";
	return $line
    }

    debug "Trying track heuristic 3";
    my @lines = split /\n/, $line;
    my @tracks = grep {/^(.*?) \[#?\d+:\d+#?\]/} @lines;
    if (@tracks) {
	$i = 1;
	for (@tracks) {
	    s/ \[#?\d+:\d+#?\]//;
	    debug "Track $i => $_";
	    push @{$tags{'tracks'}}, "d${disc}t${i}_$_";
	    $i++;
	}
    }
    else {
	debug "Heuristic 3 failed";
	return $line;
    }
    return join("\n", grep(!/^(.*?) \[#?\d+:\d+#?\]/, @lines));

=pod
    if (s/^(0*1\.?\s+.+?)(?=^\D)//sm) {
	debug "Found tracks: '$1'\n";
	$tracks = $1;
	while ($tracks =~ s/^(\d+)\.?\s+(.*?)$//sm) {
	    #print "N: $1, Name: $2\n";
	    $tags{'tracks'}[$1] = $2;
	}
    }
=cut
}

sub date_from_filename {
    my $filename = shift;
    my $year;

    #$filename =~ s/-//g;	# normalize

    if ($filename !~ /\D((?:\d{4}[-_]?\d{2}[-_]?\d{2})|(?:\d{2}[-_]?\d{2}[-_]?\d{2}))/) {
	say "date_from_filename: Unable to parse date from filename";
	return (undef, undef, undef);
    }

    my $date = $1;
    debug "Parsed: ", $date;
    $date =~ s/[-_]//g;
    $year = (length($date) == 8) ? '%Y' : '%y';
    my $t = Time::Piece->strptime($date, "$year%m%d");
    my $y = $t->year;
    $y -= 100 if $y > 2020;

    my ($mon, $day) = split(/ +/, $t->strftime("%m %d"));
    $mon += 0; $day += 0;

    $venue_date_fmt =~ s/%Y/$y/;
    return ($t->strftime("$y/%m/%d"), $t->strftime($venue_date_fmt),
	    $t->strftime(qr/[-\/_.,\s]*(?<month>%B|%b[a-z]+|%m|$mon)[-\/_.,\s]+(?<day>%d|$day)[-\/_.,\s]+(?<year>$y|%y)[-\/_., \t]*/),
	    $t->strftime(qr/[-\/_.,\s]*(?<year>$y|%y)[-\/_.,\s]+(%?<month>B|%b[a-z]+|%m|$mon)[-\/_.,\s]+(?<day>%d|$day)[-\/_., \t]*/),
	   );
}

# Compare the date parsed from the filename to the date found in the venue
sub compare_dates_venue_filename {
    my ($year,$month,$day) = @_;

    my $date = '<undetermined>';

    if ($year and $month and !$day) {
	debug "year: $year, month: $month, day: $day";
	my $fmt_y = length($year) == 4 ? '%Y' : '%y';
	my $fmt_m = '%m';
	if ($month !~ /^\d+$/) {
	    $fmt_m = '%b';
	    $month = substr($month,0,3);
	}
	debug "year fmt: $fmt_y, month fmt: $fmt_m, month: $month";

	my $t = Time::Piece->strptime("$month $day $year", "$fmt_m %d $fmt_y");
	my $y = $t->year;
	$y -= 100 if $y > 2020;
	my $date = $t->strftime("$y/%m/%d");
    }

    say "Date mismatch: from filename: '$tags{'date'}'; in info file: '$date'" if $tags{'date'} ne $date;
}

sub roman2int {
    my $r = shift;

    my %roman = (I => 1, II => 2, III => 3, IV => 4, V => 5, VI => 6, VII => 7, VIII => 8, IX => 9, X => 10);

    $r =~ s/^0+//;

    return $r =~ /[IVX]/ ? $roman{uc $r} : $r;
}

sub debug {
    return unless $opts{'d'};

    say((split(/::/, (caller(1))[3] || 'main'))[-1], ': ', @_);
}

sub do_init {
    my $infofile = shift;
    debug "**** ", $infofile;

    %tags = (venue => '', shn_id => '');
    init_bands();

    my ($base, $bandabbr) = ($infofile =~ /^((\D+).*)\.txt$/);
    $tags{'album'} = $base;
    $bandabbr =~ s/-$//;
    $tags{'artist'} = $bands{$bandabbr};
    # strip .123 or info or .textfile, ...
    debug "BASE: $base";
    if ($base =~ s/(?:\.(\d+))$//) {
	$tags{'shn_id'} = $1;
	debug "SHN_ID: $1";
    }
    $base =~ s/info|([-.][^.]+?)$//;
    debug "base: $base, band abbr: '$bandabbr', artist from lookup: '", $tags{'artist'} || '', "'";

    my (@files, @globpat, @globpat2);
    @globpat = @globpat2 = ("$base*.shn", "$base*.flac");
    map { s/(19|20)// } @globpat2;
    for my $pat (@globpat, @globpat2, '[Tr]rack*.shn') {
	debug "trying file match pattern: $pat";
	@files = bsd_glob("$pat");
	last if @files;
	debug "No music files match base of info file... trying alternatives"
    }

    say "No music files in directory - accuracy may be reduced" if ! @files;
    for (@files) {
	next unless /(?:d0*(?<disc>\d+))?t0*(?<track>\d+)/ or /0*(?<track>\d+)\./;
	my $d = $+{disc} || 1;
	debug "Found file (disc $d: track $+{track}): ", $_;
	$tags{'filenames'}{"${d}_$+{track}"} = $_;
    };

}

sub print_album {
    say "\n____Artist____\n", $tags{'artist'};
    say "\n____Album\n", $tags{'album'};
    say "\n____Date____\n", $tags{'date'};
    $tags{'venue'} .= "\n" unless substr($tags{'venue'}, -1, 1) eq "\n";
    say "\n____Venue____\n", $tags{'venue'};

    say "____Tracks____\n";
    say "Discs: ", $tags{'disc'}, "\n"  if $tags{'disc'};
    my $i = 1;
    if (exists $tags{'tracks'}) {
	for (@{$tags{'tracks'}}) {
	    my ($d,$t,$name) = /^d(\d+)t(\d+)_(.*)$/;
	    printf "%3d. %2d %2d %-40s %s\n", $i, $d, $t, $name,
		(exists($tags{'filenames'}) && $tags{'filenames'}{"$1_$2"}) ? $tags{'filenames'}{"$1_$2"} : '';
	    $i++;
	}
    }
    say "\n____SHN_ID_____\n", $tags{'shn_id'} if $tags{'shn_id'};
    say "\n____Comments____\n", $tags{'comment'}, "\n" if $tags{'comment'};
}

sub write_mpl {
    local $\ = "\r\n";

    unless (keys %{$tags{'filenames'}} and exists $tags{'tracks'} and scalar @{$tags{'tracks'}}) {
	say 'MPL not written - no files found';
	return;
    }

    sub w_property {
	my ($field, $value) = @_;

	chomp $value;
	$value =~ s/\n{3,}/\n\n/g;
	print MPLFILE '<Field Name="', $field, '">', encode_entities($value), '</Field>';

    }

    my @MCepoch = (1899, 12, 31);
    my @date = split /\//, $tags{'date'};
    my $MCdate = 1 + Delta_Days(@MCepoch, @date);

    my $mplfile = $info_file_base . '.mpl';
    open(MPLFILE, '>', $mplfile)
	or die "Unable to write mpl file $mplfile\n$!.\nStopped";

    print MPLFILE '<?xml version="1.0" encoding="UTF-8" standalone="yes" ?>';
    print MPLFILE '<MPL Version="2.0" Title="parse_etree conversion">';
    for (@{$tags{'tracks'}}) {
	if (/^d(\d+)t(\d+)_(.*)$/ and $tags{'filenames'}{"$1_$2"}) {
	    my ($d,$t,$trackname) = ($1,$2,$3);

	    my $filename = $tags{'filenames'}{"$1_$2"};
	    $filename =~ s/\.shn$/.flac/i	 if $opts{'c'};

	    my $description = join("\n", $tags{'venue'},
					 $tags{'shn_id'} ? join(' ', "SHN ID", $tags{'shn_id'}) : '');

	    print MPLFILE '<Item>';
	    w_property 'Filename',	File::Spec->catfile($cwd, $filename);
	    w_property 'Artist',	$tags{'artist'};
	    w_property 'Album',		$tags{'album'};
	    w_property 'Name',		$trackname;
	    w_property 'Date',		$MCdate;
	    w_property 'Media Type',	'Audio';
	    w_property 'Disc #',	$d    			if exists($tags{'disc'}) and $tags{'disc'} > 1;
	    w_property 'Track #',	$t;
	    w_property 'Comment',	$tags{'comment'}	if $tags{'comment'};
	    w_property 'Description',	$description		if $description;
	    print MPLFILE '</Item>';
	}
    }
    print MPLFILE '</MPL>';

    close MPLFILE;
    say "Wrote mpl file: ", $mplfile  if $opts{'v'};
}

sub do_convert {
    for (@{$tags{'tracks'}}) {
	next unless (/^d(\d+)t(\d+)_(.*)$/ and $tags{'filenames'}{"$1_$2"});

	my ($d,$t,$trackname) = ($1,$2,$3);

	my $filename = $tags{'filenames'}{"$1_$2"};
	if ($filename !~ /\.shn$/i) {
	    say 'No conversion for non-shn files: ', basename($filename);
	    next;
	}

	(my $newfilename = $filename) =~ s/\.shn$/.flac/i;
	if (!$opts{'f'} and -e $newfilename) {
	    say 'File exists - use -f to overwrite: ', $newfilename;
	    next;
	}

	my @args = ($shntool, 'conv', '-o', 'flac', '-O', $opts{'f'} ? 'always' : 'never', $filename);
	system(@args) == 0
	    or die "system @args failed: $?"
    }
}

sub init_bands {
    # note: auto-pulled from web site using pull_bands.pl
    %bands = (
	"3ah" => "3 Apples High",
	"bog" => "A Band of Gypsys",
	"jhbg" => "A Band of Gypsys",
	"ahf" => "Alan Hertz and Friends",
	"ahp" => "Alan Hertz and Friends",
	"ah&f" => "Alan Hertz and Friends",
	"abb" => "Allman Brothers Band",
	"amf" => "Amfibian",
	"bcue" => "Barbara Cue",
	"bfm" => "Barefoot Manner",
	"beanland" => "Beanlan",
	"bfft" => "Bela Fleck & the Flecktones",
	"bftt" => "Bela Fleck & Tony Trischka",
	"bh" => "Ben Harper",
	"bhic" => "Ben Harper and the Innocent Criminals",
	"bij" => "Big In Japan",
	"bs" => "Big Smith",
	"wu" => "The Big Wu",
	"bc" => "Black Crowes",
	"crowes" => "Black Crowes",
	"bmelon" => "Blind Melon",
	"bgug" => "Blueground Undergrass",
	"bt" => "Blues Traveler",
	"be" => "Bockman's Euphio",
	"bp" => "Brothers Past",
	"bruce" => "Bruce Hornsby",
	"bh" => "Bruce Hornsby",
	"buho" => "El Buho",
	"bts" => "Built to Spill",
	"bspear" => "Burning Spear",
	"bnb" => "Burt Neilson Band",
	"cvb" => "Camper van Beethoven",
	"ccity" => "Cerulean City",
	"ch" => "Charlie Hunter",
	"(see FB below)" => "Col. Claypool's Bucket of Bernie Brains",
	"cbobb" => "Col. Claypool's Bucket of Bernie Brains",
	"lcbobb" => "Col. Claypool's Bucket of Bernie Brains",
	"lcbbb" => "Col. Claypool's Bucket of Bernie Brains",
	"cc" => "Counting Crows",
	"cj" => "Cowboy Junkies",
	"cracker" => "Cracker",
	"cb" => "Critters Buggin",
	"dso" => "Dark Star Orchestra",
	"D&T" => "Dave Matthews & Tim Reynolds",
	"DM" => "Dave Matthews",
	"dmb" => "Dave Matthews Band",
	"dgray" => "David Gray",
	"dnb" => "David Nelson Band",
	"dbr" => "Day by the River",
	"dead" => "The Dead",
	"td" => "The Dead",
	"dbb" => "Deep Banana Blackout",
	"dt" => "Derek Trucks Band",
	"ddbb" => "Dirty Dozen Brass Band",
	"db" => "Disco Biscuits",
	"disp" => "Dispatch",
	"logic" => "DJ Logic",
	"djlogic" => "DJ Logic",
	"dtb" => "Donna the Buffalo",
	"dbt" => "Drive-By Truckers",
	"eb" => "Edie Brickell",
	"eh" => "Ekoostik Hookah",
	"farrah" => "Farrah",
	"fhg" => "Fareed Haque Group",
	"glove" => "G. Love and Special Sauce",
	"gal" => "Galactic",
	"garaj" => "Garaj Mahal",
	"Garcia" => "Jerry Garcia",
	"jgb" => "Jerry Garcia",
	"jg" => "Jerry Garcia",
	"porter" => "George Porter, Jr. & Runnin' Pardners",
	"gt" => "Ghost Trane",
	"gsw" => "God Street Wine",
	"mule" => "Gov't Mule",
	"gm" => "Gov't Mule",
	"gtb" => "Grand Theft Bus",
	"gd" => "Grateful Dead",
	"gba" => "GreyBoy AllStars",
	"gus" => "Guster",
	"guster" => "Guster",
	"guymalone" => "Guy Malone",
	"ht" => "Hot Tuna",
	"hday" => "Howie Day",
	"itf" => "Indiana Trip Factory",
	"ig" => "Indigo Girls",
	"jj" => "Jack Johnson",
	"jmp" => "Jazz Mandolin Project",
	"jt" => "Jeff Tweedy",
	"jgb" => "Jerry Garcia Band",
	"jmos" => "Jerry Joseph & The Jackmormons",
	"jjj" => "Jerry Joseph & The Jackmormons",
	"jhe" => "Jimi Hendrix Experience",
	"jh" => "Jimi Hendrix Experience",
	"jmayer" => "John Mayer",
	"jm3" => "John Mayer Trio",
	"jmayer3" => "John Mayer Trio",
	"sco" => "John Scofield",
	"jol" => "Jolene",
	"jk" => "Jorma Kaukonen",
	"kaikln" => "Kai Kln",
	"kk" => "Kai Kln",
	"kdcw" => "Karl Denson and Chris Wood",
	"kdtu" => "Karl Denson's Tiny Universe",
	"kw" => "Keller Williams",
	"kdub (for k-double-u)" => "Keller Williams",
	"Keller Williams w- String Cheese Incident" => "Keller Williams w/ String Cheese Incident",
	"kwi" => "Keller Williams w/ String Cheese Incident",
	"sk" => "Kimock",
	"kk" => "Kudzu Kings",
	"kudzu" => "Kudzu Kings",
	"kvhw" => "KVHW",
	"lt" => "Lake Trout",
	"ls" => "Leftover Salmon",
	"los" => "Leftover Salmon",
	"lom" => "Legion of Mary",
	"FB" => "Les Claypool's Fearless Flying Frog Brigade",
	"lcfffb" => "Les Claypool's Fearless Flying Frog Brigade",
	"claypool" => "Les Claypool's Fearless Flying Frog Brigade",
	"fffb" => "Les Claypool's Fearless Flying Frog Brigade",
	"laf" => "Life After Failing",
	"lf" => "Little Feat",
	"ld" => "Living Daylights",
	"lfb" => "Lo Faber Band",
	"mammals" => "The Mammals",
	"mel" => "Marcus Eaton & The Lobby",
	"marlow" => "Marlow",
	"mf" => "Marc Ford",
	"mmw" => "Medeski Martin & Wood",
	"mfs" => "Michael Franti & Spearhead",
	"spr" => "Michael Franti & Spearhead",
	"mcrx" => "Mike Clark's Prescription Renewal",
	"moe" => "moe.",
	"tn" => "The Nadas",
	"nd" => "The New Deal",
	"tnd" => "The New Deal",
	"nmas" => "North Mississippi Allstars",
	"oar" => "O.A.R.",
	"too" => "The Other Ones",
	"or" => "Oregon",
	"oregon" => "Oregon",
	"oh" => "Oysterhead",
	"oysterhead" => "Oysterhead",
	"par" => "Particle",
	"particle" => "Particle",
	"metheny" => "Pat Metheny",
	"pm" => "Pat Metheny",
	"pmg" => "Pat Metheny Group",
	"pmt" => "Pat Metheny Trio",
	"pmb" => "Pat McGee Band",
	"pj" => "Pearl Jam",
	"phil" => "Phil Lesh & Friends",
	"pf" => "Phil Lesh & Friends",
	"pnf" => "Phil Lesh & Friends",
	"p+f" => "Phil Lesh & Friends",
	"p&f" => "Phil Lesh & Friends",
	"paf" => "Phil Lesh & Friends",
	"ph" => "Phish",
	"pb" => "Psychedelic Breakfast",
	"rad" => "Radiators",
	"rads" => "Radiators",
	"rre" => "Railroad Earth",
	"raq" => "Raq",
	"ratdog" => "Ratdog",
	"rd" => "Ratdog",
	"rg" => "Reid Genauer",
	"reservoir" => "Reservoir",
	"rezi" => "Rezi",
	"rr" => "Robert Randolph & the Family Band",
	"rrfb" => "Robert Randolph & the Family Band",
	"rwtc" => "Robert Walter's 20th Congress",
	"rw2c" => "Robert Walter's 20th Congress",
	"rr" => "Rusted Root",
	"schas" => "Santa Cruz Hemp Allstars",
	"ho" => "Schleigho",
	"schwillb" => "The Schwillbillies",
	"amendola" => "Scott Amendola Band",
	"spod" => "1 Serial Pod",
	"Seth Yacovone Band" => "Seth Yacovone Band",
	"sy" => "Seth Yacovone Band",
	"syb" => "Seth Yacovone Band",
	"sexmob" => "Sex Mob",
	"st" => "Shaking Tree",
	"slip" => "The Slip",
	"sl" => "The Slip",
	"sy" => "Sonic Youth",
	"sonicyouth" => "Sonic Youth",
	"soulive" => "Soulive",
	"sts9" => "Sound Tribe Sector 9",
	"spin" => "Spin Doctors",
	"kmck" => "Steve Kimock & Friends",
	"skb" => "Steve Kimock Band",
	"ss" => "Stockholm Syndrome",
	"stocksyn" => "Stockholm Syndrome",
	"sf" => "Strangefolk",
	"sci" => "String Cheese Incident",
	"tlg" => "Tea Leaf Green",
	"tend" => "Tenacious D",
	"tr" => "Tim Reynolds",
	"tortoise" => "Tortoise",
	"hip" => "The Tragically Hip",
	"trey" => "Trey Anastasio",
	"tab" => "Trey Anastasio",
	"um" => "Umphrey's McGee",
	"umph" => "Umphrey's McGee",
	"us" => "Uncle Sammy",
	"vb" => "Vida Blue",
	"wh" => "Warren Haynes",
	"ween" => "Ween",
	"wsp" => "Widespread Panic",
	"wp" => "Widespread Panic",
	"wilco" => "Wilco",
	"wb4t" => "Will Bernard 4tet",
	"willyp" => "Willy Porter",
	"willyp" => "Willy Porter Band",
	"word" => "The Word",
	"ymsb" => "Yonder Mountain String Band",
	"zero" => "Zero",
	"zm" => "Zony Mash",
	"zwan" => "Zwan",
    );
}
